import os
import fileinput, re, string
import pmm_config
import logging


class ExInvalidConfigLine(Exception):
    def __init__(self, line):
        Exception.__init__(self)
        self.line = line

    def __str__(self):
        return "Invalid config line: " + self.line


_logger = logging.getLogger("pmmcli.pmmcli_config")


instance = None


def get(key = None):
    global instance
    if not instance:
        instance = PMMCliConfig()
    if not key:
        return instance
    return instance[key]


class PMMCliConfig:
    __config = {}

    def __log_info(self, keyname, exception):
        _logger.info("Exception '" + repr(exception) + "' during retrieving key " + keyname + " value: " + str(exception))

    def __init__(self, configName = None):
        if configName == None:
            configName = os.path.join(pmm_config.pmmcli_dir(),"pmmcli-rc")
        self.__parse(configName)

    def __len__(self):
        return len(self.__config)

    def __getitem__(self, key):
        return self.__config[key]

    def __iter__(self):
        return self.iterkeys()

    def iterkeys(self):
        return self.__config.iterkeys()

    def iteritems(self):
        return self.__config.iteritems()

    # free disk space required to backup operation (in bytes)
    def get_free_disk_space(self):
        free_disk_space = 1073741824
        try:
            free_disk_space = get("FREE_DISK_SPACE")
        except KeyError, e:
            self.__log_info("FREE_DISK_SPACE",e)
            pass
        return free_disk_space

    # daemon job timeout (in minutes)
    def get_daemon_timeout(self):
        daemon_timeout = 30
        try:
            daemon_timeout_str = get("DAEMON_TIMEOUT")
            daemon_timeout = int(daemon_timeout_str)
        except (KeyError, ValueError), e:
            self.__log_info("DAEMON_TIMEOUT",e)
            pass
        return daemon_timeout

    # period to keep sessions not older then (in days)
    def get_days_to_keep_sessions(self):
        days_to_keep_sessions = 30
        try:
            days_to_keep_sessions_str = get("DAYS_TO_KEEP_SESSIONS")
            days_to_keep_sessions = int(days_to_keep_sessions_str)
        except (KeyError, ValueError), e:
            self.__log_info("DAYS_TO_KEEP_SESSIONS",e)
            pass
        return days_to_keep_sessions

    def get_smtpserver(self):
        smtpserver = 'localhost'
        try:
            smtpserver = get("SMTPSERVER")
        except KeyError, e:
            self.__log_info("SMTPSERVER",e)
            pass
        return smtpserver

    def get_smtpuser(self):
        smtpuser = ''
        try:
            smtpuser = get("SMTPUSER")
        except KeyError, e:
            self.__log_info("SMTPUSER",e)
            pass
        return smtpuser

    def get_smtppass(self):
        smtppass = ''
        try:
            smtppass = get("SMTPPASS")
        except KeyError, e:
            self.__log_info("SMTPPASS",e)
            pass
        return smtppass

    def get_authrequired(self):
        authrequired = 0
        try:
            authrequired_str = get("AUTHREQUIRED")
            if authrequired_str != '0':
                authrequired = 1
        except KeyError, e:
            self.__log_info("AUTHREQUIRED",e)
            pass
        return authrequired

    def get_mailsubject(self):
        mailsubject = 'Backup/Restore task notification'
        try:
            mailsubject = get("MAILSUBJECT")
        except KeyError, e:
            self.__log_info("MAILSUBJECT",e)
            pass
        return mailsubject

    def get_mailfrom(self):
        mailfrom = ''
        try:
            mailfrom = get("MAILFROM")
        except KeyError, e:
            self.__log_info("MAILFROM",e)
            pass
        return mailfrom

    def get_mailreplyto(self):
        mailreplyto = ''
        try:
            mailreplyto = get("MAILREPLYTO")
        except KeyError, e:
            self.__log_info("MAILREPLYTO",e)
            pass
        return mailreplyto

    def get_mailbody(self):
        mailbody = 'Backup\\restore task finished.\n Owner guid is: %OWNER_GUID%\nOwner type is: %OWNER_TYPE%\n Task type is: %TASK_TYPE%\nTask status is: %TASK_STATUS%\nCreation date is: %CREATION_DATE%\n Full name is: %FULL_NAME%'
        try:
            mailbody = get("MAILBODY")
        except KeyError, e:
            self.__log_info("MAILBODY",e)
            pass
        return mailbody

    def get_max_conflict_resolves(self):
        max_conflict_resolves = 3
        try:
            max_conflict_resolves_str = get("MAX_CONFLICT_RESOLVES")
            max_conflict_resolves = int(max_conflict_resolves_str)
        except (KeyError, ValueError), e:
            self.__log_info("MAX_CONFLICT_RESOLVES",e)
            pass
        return max_conflict_resolves

    # maximum number of log files stored
    def get_max_log_files(self):
        max_max_log_files = 10
        try:
            max_max_log_files_str = get("MAX_LOG_FILES")
            max_max_log_files = int(max_max_log_files_str)
        except (KeyError, ValueError), e:
            self.__log_info("MAX_LOG_FILES",e)
            pass
        return max_max_log_files

    # maximum size of log file before rotation
    def get_max_log_size(self):
        max_log_size = 1048576
        try:
            max_log_size_str = get("MAX_LOG_SIZE")
            max_log_size = int(max_log_size_str)
        except (KeyError, ValueError), e:
            self.__log_info("MAX_LOG_SIZE",e)
            pass
        return max_log_size

    def __parse(self, configFile):
        value = re.compile('^(\S+)\s+(.*)$')
        for line in fileinput.input(configFile):
            line = line.rstrip()
            if not line:
                continue
            if line.startswith("#"):
                continue
            data = re.match(value, line)
            if not data:
                raise ExInvalidConfigLine, line
            self.__config[data.group(1)] = string.rstrip(data.group(2))


# [a sort of] testsuite
if __name__ == '__main__':
    pc = PMMCliConfig()
    for i, v in pc.iteritems():
        print i + "->"+ v
